@@ -29,33 +29,34 @@ rfc822tz(ts, prettyname)
 	offset = tz.tz_minuteswest - (ts->tm_isdst ? 60 : 0);
 	sign = offset >= 0;
 	if (!sign)
-		offset = -offset;
+	  offset = -offset;
 	sprintf(zone, "%c%02d%02d",
 		sign ? '-' : '+', offset / 60, offset % 60);
 	cp = zone;
 	if (prettyname)
-		sprintf(cp+5," (%s)",ts->tm_zone);
+	  sprintf(cp+5," (%s)",ts->tm_zone);
 
 
 #else	/* !USE_BSDTIMEZONE */
 #ifdef	USE_SYSVTIMEZONE
 	/* daylight, timezone, altzone, declared in <time.h> */
 
-	if (prettyname)
-		return tzname[!!daylight];
-
-#ifndef	_HPUX_SOURCE
+#if	!defined(_HPUX_SOURCE) && !defined(_POSIX_SOURCE)
 	if (daylight && ts->tm_isdst)
-		offset = altzone;
+	  offset = altzone;
 	else
 #endif
-		offset = timezone;
+	  offset = timezone;
 	sign = offset >= 0;
 	if (!sign)
-		offset = -offset;
+	  offset = -offset;
 	sprintf(zone, "%c%02d%02d", sign ? '-' : '+',
 		offset / (60*60), (offset % (60*60)) / 60);
 	cp = zone;
+
+	if (prettyname)
+	  strcat(zone+5, tzname[0 != daylight]);
+
 #else	/* !USE_SYSVTIMEZONE */
 #if 0
 	cp = tzname[ts->tm_isdst];	/* this doesn't always work */
