@@ -38,7 +38,7 @@ extern char *path_hash();
 extern char *progname;
 extern char *strerror();
 extern char *strnsave();
-extern char *sys_siglist[];
+extern const char * const sys_siglist[];
 extern int addbuffer();
 extern int errno;
 extern int interrupted;
@@ -941,20 +941,20 @@ runio(ioopp)
 			close(fd);
 			if (isset('R'))
 				fprintf(runiofp,
-					       "write(%d, '%.40s', %d)\n",
-					       ioop->fd, ioop->name,
-					       strlen((char *)ioop->name));
+					"write(%d, '%.40s', %d)\n",
+					ioop->fd, ioop->name,
+					strlen((char *)ioop->name));
 			break;
 		case sIOopenPipe:
 			if (pipe(p) < 0) {
 				fprintf(stderr, "%s: %s: %s\n",
-						       progname, PIPE,
-						       strerror(errno));
+					progname, PIPE,
+					strerror(errno));
 				++errflag;
 			} else if (p[1] != ioop->fd || p[0] != ioop->fd2) {
 				fprintf(stderr, "%s: pipe prediction wrong: got %d|%d not %d|%d\n",
-						progname, p[1], p[0],
-						ioop->fd, ioop->fd2);
+					progname, p[1], p[0],
+					ioop->fd, ioop->fd2);
 				++errflag;
 				close(p[1]);
 				close(p[0]);
@@ -970,9 +970,9 @@ runio(ioopp)
 			readstring(ioop->fd, ioop->command);
 			if (isset('R'))
 				fprintf(runiofp, "intoBuffer '%s'\n",
-						 ioop->command->buffer ?
-						 ioop->command->buffer->string
-						 : (u_char *)"");
+					ioop->command->buffer ?
+					ioop->command->buffer->string
+					: (u_char *)"");
 			break;
 		case sIOdup:
 /*putc('\n', runiofp);*/
@@ -1037,7 +1037,12 @@ runio(ioopp)
 					f.__fileL = ioop->fd % 256;
 					f.__fileH = ioop->fd / 256;
 #else /* Other non-portable.. */
+#ifdef	__linux__ /* and other GNU LIBC systems */
+					f._fileno = ioop->fd;
+#else
 					f._file = ioop->fd;/* XX: nonportable */
+
+#endif
 #endif
 					*(ioop->command->bufferp) = s_read(&f);
 					ioop->command->bufferp = &cdr(*ioop->command->bufferp);
