@@ -56,9 +56,9 @@ struct db_info {
 };
 
 /* bits in the flags field */
-#define	DB_MAPTOLOWER	01
-#define	DB_MAPTOUPPER	02
-#define	DB_MODCHECK	04
+#define	DB_MAPTOLOWER	0x01
+#define	DB_MAPTOUPPER	0x02
+#define	DB_MODCHECK	0x04
 
 
 struct db_kind {
@@ -138,7 +138,7 @@ run_relation(argc, argv)
 	struct db_info proto_config, *dbip;
 	struct db_kind *dbkp;
 	struct spblk *spl;
-	int c, symid, errflg, set_cache_size;
+	int c, symid, errflg, set_cache_size, copy_all_into_cache;
 	memtypes oval;
 	char *cp, *dbtype;
 
@@ -146,9 +146,10 @@ run_relation(argc, argv)
 	dbtype = NULL;
 	set_cache_size = 0;
 	optind = 1;
+	copy_all_into_cache = 0;
 	proto_config
 		= db_kinds[sizeof db_kinds/(sizeof (struct db_kind))-1].config;
-	while ((c = getopt(argc, argv, "bilmnpud:f:s:t:e:")) != EOF)
+	while ((c = getopt(argc, argv, "Cbilmnpud:f:s:t:e:")) != EOF)
 		switch (c) {
 		case 'b':	/* boolean postprocessor */
 			proto_config.postproc = Boolean;
@@ -186,6 +187,9 @@ run_relation(argc, argv)
 			proto_config.cache_size = atoi(optarg);
 			set_cache_size = 1;
 			break;
+		case 'C':	/* At open get all of it into cache */
+			copy_all_into_cache = 1;
+			break;
 		case 't':	/* database type */
 			dbtype = optarg;
 			if ((cp = strchr(optarg, ',')) != NULL
@@ -306,6 +310,9 @@ run_relation(argc, argv)
 		return 1;
 	}
 	sp_install(symid, spl->data, 0, spt_builtins);
+	if (copy_all_into_cache && dbip->cache_size > 0) {
+	  /* XX: ALL ELTS INTO CACHE! */
+	}
 	return 0;
 }
 
