@@ -45,6 +45,7 @@ static int run_process ARGCV;
 static int run_grpmems ARGCV;
 static int run_praliases ARGCV;
 static int run_listaddrs ARGCV;
+static int run_listexpand ARGCV;
 static int run_homedir ARGCV;
 static int run_822date ARGCV;
 static int run_filepriv ARGCV;
@@ -99,6 +100,7 @@ struct shCmd fnctns[] = {
 {	"groupmembers",	run_grpmems,		0,		0	},
 {	"printaliases",	run_praliases,		0,		0	},
 {	"listaddresses",run_listaddrs,		0,		0	},
+{	"listexpand",	run_listexpand,		0,		SH_ARGV	},
 {	"homedirectory",run_homedir,		0,		0	},
 {	"rfc822date",	run_822date,		0,		0	},
 {	"filepriv",	run_filepriv,		0,		0	},
@@ -1169,6 +1171,197 @@ FILE *fp;
 }
 #endif
 
+static int
+run_listexpand(argc,argv)
+	int argc;
+	char *argv[];
+{
+	struct header hs;
+	struct envelope *e;
+	struct address *ap;
+	struct token **pt, *t;
+	struct addr *pp;
+	int c, n, errflag, stuff;
+	char *comment, *erroraddress;
+	u_char *s, *s_end;
+	struct siobuf *osiop;
+	FILE *mfp, *fp, f;
+	char buf[4096];
+	int fd2;
+
+	errflag = 0;
+	erroraddress = NULL;
+	comment = "list";
+	optind = 1;
+
+	while ((c = getopt(argc, argv, "c:e:E:")) != EOF) {
+		switch (c) {
+		  case 'c':
+		    comment = optarg;
+		    break;
+		  case 'e':
+		    erroraddress = optarg;
+		    break;
+		  case 'E':
+		    if (errors_to) free(errors_to);
+		    errors_to = (void*)malloc(strlen(optarg)+1);
+		    if (errors_to)
+		      strcpy(errors_to, optarg);
+		    break;
+		  default:
+		    ++errflag;
+		    break;
+		}
+	}
+	if (errflag) {
+		fprintf(stderr,
+			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [ -c comment ] $attribute $localpart $origaddr < /file/path \n",
+			argv[0]);
+		return 1;
+	}
+	/* We (memopry-)leak this stuff for a moment.. */
+	e = (struct envelope *)tmalloc(sizeof (struct envelope));
+	e->e_nowtime = now;
+	e->e_file = argv[0];
+	/* we only need sensible `e' if its a time stamp header,
+	   which it ain't, so "listaddress" is just fine for e_file .. */
+
+	hs.h_descriptor = &aliashdr;
+	hs.h_pname = argv[0];
+
+	/* create h->h_lines */
+	/*
+	 * It is best to maintain a line at a time as tokens, so errors will
+	 * print out nicely.
+	 */
+	initline(4096);
+	pt = &hs.h_lines;
+	t = NULL;
+	/*
+	 * These hoops are so we can do this for both real stdin and
+	 * the fake I/O stuff in ../libsh.
+	 */
+	if ((fp = fdopen(0, "r")) == NULL) {
+		fprintf(stderr, "%s: fdopen failed\n", argv[0]);
+#ifdef DEBUG_FOPEN
+		report_fds(stderr);
+#endif
+		return 1;
+	}
+	fd2 = dup(0);	/* Copy the file handle for latter returning
+			   AFTER an fclose() has closed the primary
+			   copy.. */
+
+	/* use a constant buffer to avoid memory leaks from stdio usage */
+	setvbuf(fp, buf, _IOFBF, sizeof buf);
+	c = 0;
+	while ((n = getline(fp)) > 0) {
+		/*
+		 * For sendmail compatibility, addresses may not cross line
+		 * boundaries, and line boundary is just as good an address
+		 * separator as comma is, sigh.
+		 */
+		if (linebuf[n-1] == '\n')
+			--n;
+		stuff = 0;
+		s_end = &linebuf[n];
+		for (s = linebuf; s < s_end; ++s) {
+			if (isascii(*s) && !isspace(*s))
+				c = stuff = 1;
+			if (*s == ',')
+				c = 0;
+		}
+		if (!stuff)
+			continue;
+		if (c) {
+			linebuf[n] = ',';
+			++n;
+		}
+
+/* #ifdef SENDMAIL_COMPABILITY_KLUDGE */
+		/**
+		 * Additional sendmail compability kludge ++haa
+		 * If address starts with \, zap the \ character.
+		 * This is just to not to force people to edit their
+		 * .forwards :-(  "\haa" works as expected  :-)
+		 **/
+		if (linebuf[0] == '\\') linebuf[0] = ' ';
+		for (s = linebuf+1; s < s_end; ++s) {
+		  if (s[-1] == ',' && s[0]=='\\') s[0] = ' ';
+		}
+		/* end of sendmail compatibility kluge */
+/* #endif */
+
+		*pt = t = makeToken(linebuf, n);
+		t->t_type = Line;
+		pt = &(t->t_next);
+	}
+
+	fclose(fp);	/* Now we discard the stdio buffers, but not the
+			   fd number 0!  Actually we use a copy of it..	*/
+	dup2(fd2,0);	/* Return the fd to descriptor 0..		*/
+	close(fd2);	/* .. and discard the backup copy..		*/
+
+	if (t == NULL) {	/* if the file is empty, use error address */
+		if (erroraddress == NULL)
+			erroraddress = "postmaster";
+
+		*pt = t = makeToken(erroraddress, strlen(erroraddress));
+		t->t_type = Line;
+		pt = &(t->t_next);
+	}
+
+	/* fix up any trailing comma (more sendmail compatibility kluges) */
+
+	s = t->t_pname + TOKENLEN(t)-1;
+	if (c && (*s == ',' || *s == '\n'))
+		*s = '\0';
+	*pt = NULL;
+
+	hs.h_contents = hdr_scanparse(e, &hs, 1);
+	hs.h_stamp = hdr_type(&hs);
+	if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
+	  if (hs.h_stamp == BadHeader) {
+	    if (erroraddress != NULL) {
+	      if (!iserrmessage()
+		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
+		osiop = siofds[fileno(mfp)];
+		siofds[fileno(mfp)] = NULL;
+		fprintf(mfp, "channel error\n");
+		fprintf(mfp, "To: %s\n", erroraddress);
+		fprintf(mfp, "Subject: Error in %s\n\n", comment);
+		hdr_errprint(e, &hs, mfp, comment);
+		siofds[fileno(mfp)] = osiop;
+		mail_close(mfp);
+	      }
+	      printf("%s\n", erroraddress);
+	    }
+	    hdr_errprint(e, &hs, stderr, comment);
+	  } else {		/* if (hs.h_contents.a == NULL) */
+	    if (erroraddress != NULL)
+	      printf("%s\n", erroraddress);
+	    fprintf(stderr, "%s: null input\n", argv[0]);
+	  }
+	  return 1;
+	}
+	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
+		for (pp = ap->a_tokens; pp != NULL; pp = pp->p_next)
+			if (pp->p_type == anAddress)
+				break;
+		if (pp == NULL)
+			continue;
+		pureAddress(stdout, ap->a_tokens);
+		/* printAddress(stdout, ap->a_tokens, 0); */
+		putc('\n', stdout);
+	}
+	/*
+	 * XX: If the alias expansion was a mail group,
+	 * we didn't print anything.
+	 */
+	return 0;
+
+} /* end-of: run_listexpand() */
+
 static int
 run_listaddrs(argc, argv)
 	int argc;
@@ -1301,7 +1494,7 @@ run_listaddrs(argc, argv)
 
 	if (t == NULL) {	/* if the file is empty, use error address */
 		if (erroraddress == NULL)
-		  erroraddress = "postmaster";
+			erroraddress = "postmaster";
 
 		*pt = t = makeToken(erroraddress, strlen(erroraddress));
 		t->t_type = Line;
@@ -1318,29 +1511,28 @@ run_listaddrs(argc, argv)
 	hs.h_contents = hdr_scanparse(e, &hs, 1);
 	hs.h_stamp = hdr_type(&hs);
 	if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
-		if (hs.h_stamp == BadHeader) {
-			if (erroraddress != NULL) {
-				if (!iserrmessage()
-				    && (mfp = mail_open(MSG_RFC822)) != NULL) {
-					osiop = siofds[fileno(mfp)];
-					siofds[fileno(mfp)] = NULL;
-					fprintf(mfp, "channel error\n");
-					fprintf(mfp, "To: %s\n", erroraddress);
-					fprintf(mfp, "Subject: Error in %s\n\n",
-							comment);
-					hdr_errprint(e, &hs, mfp, comment);
-					siofds[fileno(mfp)] = osiop;
-					mail_close(mfp);
-				}
-				printf("%s\n", erroraddress);
-			}
-			hdr_errprint(e, &hs, stderr, comment);
-		} else {/* if (hs.h_contents.a == NULL) */
-			if (erroraddress != NULL)
-				printf("%s\n", erroraddress);
-			fprintf(stderr, "%s: null input\n", argv[0]);
-		}
-		return 1;
+	  if (hs.h_stamp == BadHeader) {
+	    if (erroraddress != NULL) {
+	      if (!iserrmessage()
+		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
+		osiop = siofds[fileno(mfp)];
+		siofds[fileno(mfp)] = NULL;
+		fprintf(mfp, "channel error\n");
+		fprintf(mfp, "To: %s\n", erroraddress);
+		fprintf(mfp, "Subject: Error in %s\n\n", comment);
+		hdr_errprint(e, &hs, mfp, comment);
+		siofds[fileno(mfp)] = osiop;
+		mail_close(mfp);
+	      }
+	      printf("%s\n", erroraddress);
+	    }
+	    hdr_errprint(e, &hs, stderr, comment);
+	  } else {		/* if (hs.h_contents.a == NULL) */
+	    if (erroraddress != NULL)
+	      printf("%s\n", erroraddress);
+	    fprintf(stderr, "%s: null input\n", argv[0]);
+	  }
+	  return 1;
 	}
 	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
 		for (pp = ap->a_tokens; pp != NULL; pp = pp->p_next)
@@ -1410,10 +1602,18 @@ run_filepriv(argc, argv)
 	int id;
 	struct stat stbuf;
 	char *file, *cp, *dir;
+	int maxperm = 0666 ^ filepriv_mask_reg; /* XOR.. */
 	FILE *fp;
 
-	if (argc == 1 || argc > 3) {
-		fprintf(stderr, "Usage: %s pathname [ uid ]\n", argv[0]);
+	if (argc > 2 && argv[1][0] == '-' && argv[1][1] == 'M') {
+	  ++argv;
+	  --argc;
+	  maxperm = (int)strtol(argv[1],NULL,8); /* Octal! */
+	  ++argv;
+	  --argc;
+	}
+	if (argc == 1 || argc > 3 || (maxperm & (~0664)) != 0) {
+		fprintf(stderr, "Usage: %s [-M maxperm] pathname [ uid ]\n", argv[0]);
 		return 1;
 	}
 	file = argv[1];
@@ -1434,12 +1634,11 @@ run_filepriv(argc, argv)
 			return 3;
 		}
 		fclose(fp);
-		if ((stbuf.st_mode & S_IFMT) != S_IFREG
-		    || ((stbuf.st_mode & filepriv_mask_reg)
-			&& !(stbuf.st_mode & S_ISVTX))) {
+		if (!S_ISREG(stbuf.st_mode)
+		    || ((stbuf.st_mode & 07777) & ~maxperm) != 0) {
 			/*
-			 * If it is a symlink or special or directory
-			 * or writable by non-owner, don't trust it
+			 * If it is a  special or directory or writable
+			 * by non-owner (modulo permission), don't trust it!
 			 */
 			printf("%d\n", nobody);
 			return 0;
@@ -1459,7 +1658,7 @@ run_filepriv(argc, argv)
 	bcopy(file, dir, cp - file);
 	dir[cp - file] = '\0';
 
-	if (stat(dir, &stbuf) < 0 || (stbuf.st_mode & S_IFMT) != S_IFDIR) {
+	if (stat(dir, &stbuf) < 0 || !S_ISDIR(stbuf.st_mode)) {
 		fprintf(stderr, "%s: not a directory: \"%s\"!\n",
 			argv[0], dir);
 #ifndef	USE_ALLOCA
