@@ -55,6 +55,9 @@ extern long offset();
 extern void initline();
 
 
+static void	prctladdr __((struct conscell *info, FILE *fp, int cfflag, char *comment));
+static void	prdsndata __((struct conscell *info, FILE *fp, int cfflag, char *comment));
+
 /*
  * Apply RFC822 parsing and processing to the message in the file in argv[1].
  */
@@ -78,8 +81,8 @@ run_rfc822(argc, argv)
 			break;
 		}
 	}
+	if (errflg || optind != (argc - 1)) { 
 */
-	/* if (errflg || optind != (argc - 1)) { */
 	if (argc != 2) {
 		fprintf(stderr, "Usage: %s messagefile\n", argv[0]);
 		return PERR_USAGE;
@@ -361,6 +364,8 @@ dumpHeader(h)
 			printf("\tVia: %s\n", formatToken(t));
 		for (t = h->h_contents.r->r_with; t != NULL; t = t->t_next)
 			printf("\tWith: %s\n", formatToken(t));
+		if ((t = h->h_contents.r->r_convert) != NULL)
+			printf("\tConvert: %s\n", formatToken(t));
 		printf("\tUNIX time %d, local time %s",
 			h->h_contents.r->r_time,
 			ctime(&(h->h_contents.r->r_time)));
@@ -548,20 +553,22 @@ reject(e, msgfile)
 	h->h_pname = cp;
 	lastCh = nextToLastCh = '\0';
 	setvbuf(fp, vbuf, _IOFBF, sizeof vbuf);
-	while ((n = fread(buf, 1, sizeof buf, fp)) > 0) {
-		if (fwrite(buf, 1, n, mfp) != n) {
-			squirrel(e, "Couldn't include reject message file");
-			fclose(fp);
-			mail_abort(mfp);
-			return;
-		}
-		if (n > 1) {
-			nextToLastCh = buf[n-2];
-			lastCh = buf[n-1];
-		} else {
-			nextToLastCh = lastCh;
-			lastCh = buf[0];
-		}
+	while (fgets(buf,sizeof(buf),fp) != NULL) {
+	  int rc;
+	  n = strlen(buf);
+	  if (strncmp("HDR",buf,3)==0 ||
+	      strncmp("SUB",buf,3)==0) {
+	    rc = fputs(buf+4,mfp);
+	    n -= 4;
+	  } else
+	    rc = fputs(buf,mfp);
+	  if (n > 1) {	  
+	    nextToLastCh = buf[n-2];
+	    lastCh = buf[n-1];
+	  } else {
+	    nextToLastCh = lastCh;
+	    lastCh = buf[0];
+	  }
 	}
 	fclose(fp);
 	if (lastCh == '\n') {
@@ -678,6 +685,7 @@ mkSender(e, name, flag)
 	sh->h_contents.a->a_tokens = NULL;
 	sh->h_contents.a->a_next = NULL;
 	sh->h_contents.a->a_stamp = newAddress;
+	sh->h_contents.a->a_dsn   = NULL;
 	ppp = &sh->h_contents.a->a_tokens;
 	didbracket = 0;
 	FindEnvelope(eFullname);
@@ -819,6 +827,7 @@ mkTrace(e)
 		/* we don't need to fill in the other structure elements */
 		/* but, just in case... */
 		ap->a_stamp = newAddress;
+		ap->a_dsn  = NULL;
 		th->h_contents.r->r_by = ap;
 	} else
 		th->h_contents.r->r_by = NULL;
@@ -842,6 +851,7 @@ mkTrace(e)
 	ap = (struct address *)tmalloc(sizeof (struct address));
 	ap->a_tokens = na;
 	ap->a_next = NULL;
+	ap->a_dsn  = NULL;
 	ap->a_stamp = newAddress;
 	th->h_contents.r->r_id = ap;
 	/* for */
@@ -856,6 +866,7 @@ struct rwmatrix {
 	struct rwmatrix	*next;
 	struct rwmatrix	*down;
 	struct conscell *info;	/* rewrite, sender, or recipient list info */
+	struct notary   *dsn;
 	union {
 		int	number;	/* XOR address id number for recipients */
 		struct header *h;	/* for rewritings */
@@ -870,6 +881,7 @@ rwalloc(rwpp)
 
 	p = (struct rwmatrix *)tmalloc(sizeof (struct rwmatrix));
 	p->down = NULL;
+	p->dsn  = NULL;
 	p->urw.number = 0;			/* unused but why not */
 	if (*rwpp != NULL)
 		p->next = *rwpp;
@@ -979,13 +991,14 @@ sequencer(e, file)
 	struct address *a, *ap;
 	struct addr *p;
 	char *ofpname, *path, *qpath;
-	struct conscell *l, *routed_addresses, *sender, *to, *x, *tmp;
+	struct conscell *l, *routed_addresses, *sender, *to;
 	struct rwmatrix *rwhead, *nsp, *rwp = NULL, *rcp = NULL;
 	struct token *t = NULL;
 	int idnumber, nxor, i;
 	int def_uid, header_error, perr;
 	FILE *ofp, *vfp;
 	char vbuf[2048], verbosefile[1024];
+	char *envid = NULL;
 
 
 	if (e == NULL)
@@ -1064,6 +1077,12 @@ sequencer(e, file)
 	if (perr)
 		return perr;
 
+	FindEnvelope(eEnvid);
+	envid = NULL;
+	if (h != NULL && h->h_contents.a != NULL
+	    && h->h_contents.a->a_pname != NULL) {
+		envid = h->h_contents.a->a_pname;
+	}
 	FindEnvelope(eFrom);
 	if (h == NULL) {
 		dprintf("A sender was NOT specified in the envelope\n");
@@ -1373,17 +1392,39 @@ sequencer(e, file)
 	if (errors_to) free(errors_to);
 	errors_to = NULL;
 	for (h = e->e_eHeaders; h != NULL; h = h->h_next) {
-		if (h->h_descriptor->class != eTo)
+		static struct notary *DSN;
+		if (h->h_descriptor->class != eTo &&
+		    h->h_descriptor->class != eToDSN) {
+			DSN = NULL;
 			continue;
+		}
 		if (D_sequencer) dumpHeader(h);
+		if (h->h_descriptor->class == eToDSN) {
+			DSN = NULL;
+			if (h->h_contents.a == NULL ||
+			    h->h_contents.a->a_pname == NULL ||
+			    h->h_contents.a->a_pname[0] == 0) {
+			  /* HUH!  How come ? "todsn" w/o data ?? */
+			  continue;
+			}
+			DSN = (struct notary *)tmalloc(sizeof(struct notary));
+			DSN->envid = envid;
+			DSN->dsn = h->h_contents.a->a_pname;
+			continue;
+		}
+		if (DSN != NULL) {
+			h->h_contents.a->a_dsn = DSN;
+			DSN = NULL;
+		}
 		for (a = h->h_contents.a; a != NULL; a = a->a_next) {
 			l = router(a, def_uid, "recipient");
 			if (l == NULL)
 				continue;
 
-			if (routed_addresses == NULL)
+			if (routed_addresses == NULL) {
+				struct conscell *tmp;
 				routed_addresses = ncons(car(l));
-			else {
+			} else {
 				cdr(s_last(car(l))) = car(routed_addresses);
 				car(routed_addresses) = car(l);
 			}
@@ -1393,7 +1434,10 @@ sequencer(e, file)
 
 	dprintf("Crossbar applied to all (sender,routed-address) pairs\n");
 
-	sender = ncons(e->e_from_trusted);
+	{
+	  struct conscell *tmp;
+	  sender = ncons(e->e_from_trusted);
+	}
 #if 0
 	if (LIST(sender) && LIST(car(sender)))
 		sender = caar(sender);
@@ -1407,6 +1451,8 @@ sequencer(e, file)
 		/* first level is ANDs */
 		++idnumber, nxor = 0;
 		for (to = car(l); to != NULL; to = cdr(to)) {
+			struct conscell *x, *tmp;
+
 			/* secondlevel is XORs */
 #if 0
 			printf("crossbar sender: ");
@@ -1468,7 +1514,8 @@ sequencer(e, file)
 					s_grind(nsp->info, stdout);
 					printf("\n");
 					fflush(stdout);
-					for (rp = nsp->down; rp != NULL; rp = rp->next) {
+					for (rp = nsp->down; rp != NULL;
+					     rp = rp->next) {
 						printf("**\t\t");
 						s_grind(rp->info, stdout);
 						printf("\n");
@@ -1514,11 +1561,14 @@ sequencer(e, file)
 #ifdef	USE_ALLOCA
 	ofpname = (char *)alloca((u_int)(strlen(file)+2));
 #else
-	ofpname = (char *)tmalloc((u_int)(strlen(file)+2));
+	ofpname = (char *)emalloc((u_int)(strlen(file)+2));
 #endif
 	ofpname[0] = '.';
 	strcpy(&ofpname[1], file);
 	if ((ofp = fopen(ofpname, "w+")) == NULL) {
+#ifndef USE_ALLOCA
+		free(ofpname);
+#endif
 		return PERR_CTRLFILE;
 	}
 	setvbuf(ofp, vbuf, _IOFBF, sizeof vbuf);
@@ -1530,6 +1580,9 @@ sequencer(e, file)
 		_CF_MESSAGEID, _CFTAG_NORMAL, file);
 	fprintf(ofp, "%c%c%d\n",
 		_CF_BODYOFFSET, _CFTAG_NORMAL, e->e_msgOffset);
+	if (envid != NULL)
+		fprintf(ofp, "%c%c%s\n",
+			_CF_DSNENVID, _CFTAG_NORMAL, envid);
 	if (e->e_messageid != NULL)
 		fprintf(ofp, "%c%c%s\n",
 			_CF_LOGIDENT, _CFTAG_NORMAL, e->e_messageid);
@@ -1542,7 +1595,8 @@ sequencer(e, file)
 	 */
 
 	if (!iserrmessage()) {
-		if (errors_to != NULL) {	/* [mea@utu.fi] Stupid, but workable.. */
+		if (errors_to != NULL) {	/* [mea@utu.fi] Stupid,
+						   but workable.. */
 			putc(_CF_ERRORADDR, ofp);
 			putc(_CFTAG_NORMAL, ofp);
 			fprintf(ofp,"%s\n",errors_to);
@@ -1564,7 +1618,8 @@ sequencer(e, file)
 	}
 
 	/*
-	 * If this message might obsolete another we need to tell the scheduler.
+	 * If this message might obsolete another we need to tell
+	 * the scheduler.
 	 */
 
 	for (h = e->e_headers; h != NULL; h = h->h_next) {
@@ -1627,11 +1682,11 @@ sequencer(e, file)
 		for (nsp = rwp->down; nsp != NULL; nsp = nsp->next) {
 			/* print envelope sender address */
 			fprintf(ofp, "%c%c", _CF_SENDER, _CFTAG_NORMAL);
-			prctladdr(nsp->info, ofp, "sender");
+			prctladdr(nsp->info, ofp, _CF_SENDER, "sender");
 			putc('\n', ofp);
 			if (vfp != NULL) {
 				fprintf(vfp, "%c%c", _CF_SENDER, _CFTAG_NORMAL);
-				prctladdr(nsp->info, vfp, "sender");
+				prctladdr(nsp->info, vfp, _CF_SENDER, "sender");
 				putc('\n', vfp);
 			}
 			/* print recipient addresses */
@@ -1643,8 +1698,10 @@ sequencer(e, file)
 				putc(_CFTAG_NORMAL, ofp);
 				if (rcp->urw.number > 0)
 					fprintf(ofp, "%d ", rcp->urw.number);
-				prctladdr(rcp->info, ofp, "recipient");
+				prctladdr(rcp->info, ofp, _CF_RECIPIENT, "recipient");
 				putc('\n', ofp);
+				/* XX: DSN data output ! */
+				prdsndata(rcp->info, ofp, _CF_RCPTNOTARY, "recipient");
 				if (vfp != NULL) {
 					if (rcp->urw.number > 0)
 						fprintf(vfp, "%c%c%d ",
@@ -1655,7 +1712,7 @@ sequencer(e, file)
 						fprintf(vfp, "%c%c",
 							     _CF_RECIPIENT,
 							     _CFTAG_NORMAL);
-					prctladdr(rcp->info, vfp, "recipient");
+					prctladdr(rcp->info, vfp, _CF_RECIPIENT, "recipient");
 					putc('\n', vfp);
 				}
 			}
@@ -1773,10 +1830,11 @@ sequencer(e, file)
 	return PERR_OK;
 }
 
-void
-prctladdr(info, fp, comment)
+static void
+prctladdr(info, fp, cfflag, comment)
 	struct conscell *info;
 	FILE *fp;
+	int cfflag;
 	char *comment;
 {
 	int i = 0;
@@ -1812,3 +1870,36 @@ prctladdr(info, fp, comment)
 			fprintf(stderr, "Malformed %s\n", comment);
 	}
 }
+
+static void
+prdsndata(info, fp, cfflag, comment)
+	struct conscell *info;
+	FILE *fp;
+	int cfflag;
+	char *comment;
+{
+	int i = 0;
+	register struct conscell *l, *x = NULL;
+
+	for (l = car(info); l != NULL; l = cdr(l)) {
+	  ++i;
+	  if (STRING(l)) {
+	    if (cdr(l) == NULL
+		&& (x = v_find(l->string)) != NULL
+		&& (x = cdr(x)) != NULL
+		&& LIST(x)) {
+	      for (x = car(x); x != NULL; x = cddr(x)) {
+		if (STRING(x) && strcmp((char *)x->string,"DSN") == 0) {
+		  x = cdr(x);
+		  break;
+		}
+	      }
+	      /* if x == NULL, no DSN was specified */
+	    }
+	  } else
+	    fprintf(stderr, "Malformed DSN %s\n", comment);
+	}
+	if (x != NULL)
+	  if (*x->string != '\0')
+	    fprintf(fp, "%c %s\n", cfflag, x->string);
+}
