@@ -7,11 +7,11 @@
  * ZMailer transport scheduler.
  */
 
+#include <sys/param.h>
 #include "hostenv.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <sys/param.h>
 #include "scheduler.h"
 #include <sys/stat.h>
 #include <signal.h>
@@ -20,6 +20,10 @@
 #include <mail.h>
 #include NDIR_H
 
+#ifndef	MAXNAMLEN /* POSIX.1 ... */
+#define MAXNAMLEN NAME_MAX
+#endif
+
 #ifndef	_IOLBF
 # define _IOLBF 0200
 #endif	/* !_IOLBF */
