@@ -3,15 +3,15 @@
  *	This will be free software, but only when it is finished.
  */
 
-#include "hostenv.h"
 #include <stdio.h>
 #include <sys/param.h>
+#include "hostenv.h"
 #include "scheduler.h"
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <sys/file.h>
-
+#include NDIR_H
 /*
  * FD_SETSIZE should be set in either <sys/param.h> or
  * <sys/types.h> if we have USE_SELECT
@@ -38,6 +38,7 @@ extern char *progname;
 extern char *replchannel;
 extern char *replhost;
 extern char *vtx_to_dir();
+extern char *strchr();
 extern int cistrcmp(), ranny();
 extern int epipe();
 extern int errno;
