@@ -71,6 +71,7 @@ char *Copyright2= "Copyright 1991-1994 Matti Aarnio";
 #endif /* !ULTRIX */
 #endif /* !SYSV */
 #endif /* !USE_STATVFS */
+
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
 #endif	/* USE_UNIONWAIT */
@@ -806,15 +807,21 @@ int insecure;
 		type(452, (char *)NULL);
 		return;
 	}
-#ifdef	FSTATFS
+#if defined(__linux__)
+	if ((rc = fstatfs(fileno(mfp), &statbuf)) == 0) {
+	  availspace = statbuf.f_bavail * statbuf.f_bsize;
+	}
+#else
+#if defined(FSTATFS)
 	if ((rc = FSTATFS(fileno(mfp), &statbuf)) == 0) {
 	  availspace = statbuf.f_bavail * statbuf.f_frsize;
 	}
-#else
+#else /* no __linux__, no FSTATFS */
 	if ((rc = fstatfs(fileno(mfp),
 			  &statbuf,sizeof statbuf,0)) == 0){
 	  availspace = statbuf.f_bfree * statbuf.f_bsize;
 	}
+#endif
 #endif
 	if (availspace < 0)
 	  availspace = 2000000000; /* Over 2G ? */
