@@ -12,6 +12,7 @@
 /*#include <string.h>*/
 #include <sys/param.h>
 #include "malloc.h"
+#include NDIR_H
 
 #if	defined(USE_HOSTS) || defined(USE_RESOLV)
 #include <netdb.h>
@@ -210,8 +211,27 @@ process(dp)
 	  if (rp->status == EX_OK) {
 	    rp->addr->user = cp;
 	    sawok = 1;
-	  } else
+	  } else {
+	    char *action = NULL;
+	    char *status = NULL;
+	    switch (rp->status) {
+	    case EX_PROTOCOL:
+	    case EX_SOFTWARE:
+	      action = "failed";
+	      status = "500 (Protocol failure inside ZMAILER! AARGH!)";
+	      break;
+	    case EX_TEMPFAIL:
+	      action = "delayed";
+	      status = "466 (Temporary routing lookup failure)";
+	      break;
+	    default:
+	      action = "failed";
+	      status = "500 (Unknown HOLD return code **CAN'T HAPPEN**)";
+	      break;
+	    }
+	    notaryreport(rp->addr->user,action,status);
 	    diagnostic(rp, rp->status, "%s", rp->addr->user);
+	  }
 	}
 
 	if (!sawok)
@@ -227,6 +247,8 @@ process(dp)
 	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	    if (rp->status == EX_OK) {
+	      notaryreport(rp->addr->user,"delayed",
+			   "400 (Cannot resubmit anything, out of spool space?)");
 	      diagnostic(rp, EX_TEMPFAIL,
 			 "cannot resubmit anything!");
 	    }
@@ -240,10 +262,39 @@ process(dp)
 #endif	/* USE_SETREUID */
 
 	fprintf(mfp, "via suspension\n");
-	fprintf(mfp, "from <%s>\n", dp->senders->user);
+	if (dp->senders->user[0] == 0)
+	  fprintf(mfp, "channel error\n");
+	else
+	  fprintf(mfp, "from <%s>\n", dp->senders->user);
+	if (dp->envid != NULL)
+	  fprintf(mfp, "envid %s\n", dp->envid);
+	
 	for (rp = dp->recipients; rp != NULL; rp = rp->next)
-	  if (rp->status == EX_OK)
+	  if (rp->status == EX_OK) {
 	    fprintf(mfp, "to <%s>\n", rp->addr->user);
+	    if (rp->dsnflags != 0 || rp->orcpt != NULL) {
+
+	      fputs("todsn",mfp);
+	      if (rp->orcpt != NULL)
+		fprintf(mfp," ORCPT=%s",rp->orcpt);
+
+	      if (rp->dsnflags & _DSN_RET_YES)
+		fputs(" RET=YES",mfp);
+	      else if (rp->dsnflags & _DSN_RET_NO)
+		fputs(" RET=NO",mfp);
+
+	      if (rp->dsnflags & _DSN_NOTIFY_SUCCESS)
+		fputs(" NOTIFY=SUCCESS",mfp);
+	      else if (rp->dsnflags & _DSN_NOTIFY_FAILURE)
+		fputs(" NOTIFY=FAILURE",mfp);
+	      else if (rp->dsnflags & _DSN_NOTIFY_ALWAYS)
+		fputs(" NOTIFY=ALWAYS",mfp);
+	      else if (rp->dsnflags & _DSN_NOTIFY_NEVER)
+		fputs(" NOTIFY=NEVER",mfp);
+
+	      putc('\n',mfp);
+	    }
+	  }
 
 	writeheaders(dp->recipients,mfp,"\n",0,0);
 
@@ -264,8 +315,11 @@ process(dp)
 	}
 
 	for (rp = dp->recipients; rp != NULL; rp = rp->next)
-	  if (rp->status == EX_OK)
+	  if (rp->status == EX_OK) {
+	    notaryreport(rp->addr->user,"relayed",
+			 "651 (Relayed via deferral channel)");
 	    diagnostic(rp, code, cp);
+	  }
 }
 
 
