@@ -42,6 +42,7 @@
 #include "ta.h"
 
 extern int cistrcmp();
+extern char *routermxes();
 
 static struct address *ctladdr __((char *cp));
 
@@ -107,17 +108,14 @@ ctlclose(dp)
 	  free((char *)dp->offset);
 	for (ap = dp->senders; ap != NULL; ap = nextap) {
 	  nextap = ap->link;
-	  ap->link = NULL; /* XX: Excessive caution.. */
 	  free((char *)ap);
 	}
 	dp->senders = NULL; /* XX: Excessive caution.. */
 	for (rp = dp->recipients; rp != NULL; rp = nextrp) {
 	  nextrp = rp->next;
-	  rp->next = NULL; /* XX: Excessive caution.. */
 	  ap = rp->addr;
 	  if (ap->routermxes) {
 	    free((char *)ap->routermxes);
-	    ap->routermxes = NULL; /* XX: Excessive caution.. */
 	  }
 	  free((char *)rp);
 	}
