@@ -20,8 +20,8 @@ warning(fmt, arg)
 
 	if (progname != NULL)
 		(void) fprintf(stderr, "%s: ", progname);
-	(void) fprintf(stderr, fmt, arg);
-	(void) fprintf(stderr, "\n");
+	fprintf(stderr, fmt, arg);
+	fprintf(stderr, "\n");
 #if	defined(BSD) || defined(sun)
 	if (!isopen) {
 #ifdef	LOG_MAIL
@@ -31,7 +31,7 @@ warning(fmt, arg)
 #endif	/* LOG_MAIL */
 		isopen = 1;
 	}
-	(void) syslog(LOG_ERR, fmt, arg);
+	syslog(LOG_ERR, fmt, arg);
 #else	/* SVID */
 	isopen = isopen;	/* appease lint */
 #endif	/* BSD */
