@@ -9,7 +9,7 @@
  * Rewritten for Sun environment by Dennis Ferguson and Rayan Zachariassen.
  * RBIFF code originally added by Jean-Francois Lamy.
  * Heavily modified for ZMailer by Rayan Zachariassen.
- *
+ * Still more modifications by Matti Aarnio <mea@nic.funet.fi>
  */
 
 #include <stdio.h>
@@ -91,6 +91,8 @@ struct utimbuf {
 #define	F_OK	0
 #endif	/* !F_OK */
 
+#include NDIR_H
+
 #ifdef	BIFF
 #include <netdb.h>
 #include <sys/socket.h>
@@ -102,7 +104,6 @@ struct utimbuf {
 #endif	/* BIFF */
 
 #ifdef RBIFF
-#include NDIR_H
 #include <utmp.h>
 #include <protocols/rwhod.h>
 #define RWHODIR		"/var/spool/rwho"
@@ -206,6 +207,11 @@ int creatflag = 1;		/* attempt to create files that don't exist */
 int mime8bit = 0;		/* Has code which translates  MIME text/plain
 				   Quoted-Printable  to 8BIT.. */
 int convert_qp = 0;		/* Flag: We have a job.. */
+int is_mime    = 0;		/* Flag: Msg is MIME msg .. */
+char *mime_boundary = NULL;
+int   mime_boundary_len = 0;
+int mmdf_mode = 0;		/* Write out MMDF-style mail folder
+				   ("\001\001\001\001" as the separator..)  */
 
 extern SIGNAL_TYPE wantout();
 extern time_t time();
@@ -222,7 +228,7 @@ extern FILE *putmail();
 extern void program(), setrootuid();
 extern int appendlet();
 extern char **environ;
-extern int exstat(), setupuidgid(), createfile();
+extern int exstat();
 extern int writebuf();
 
 extern int  qp_to_8bit   __((struct rcpt *rp));
@@ -283,11 +289,14 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gl:rV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:gl:rMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
 	    break;
+	  case 'M':
+	    mmdf_mode = 1;
+	    break;
 	  case 'V':
 	    prversion(PROGNAME);
 	    exit(EX_OK);
@@ -671,7 +680,7 @@ deliver(rp, dp, timestring)
 #endif	/* USE_NFSMBOX */
 #ifdef	USE_LOCKF
 	  /* Seek to begining for locking! [mea@utu.fi] */
-	  if (lseek(fdmail,0,0) < 0 ||
+	  if (lseek(fdmail,0L,L_SET) < 0 ||
 	      lockf(fdmail, F_LOCK, 0) < 0) {
 	    alarm(0);
 	    notaryreport(file,"delayed","450 (File locking with  lockf  failed)");
@@ -748,16 +757,18 @@ deliver(rp, dp, timestring)
 	  nbp->offset = lseek(fdmail, 0L, L_XTND);
 #endif	/* BIFF || RBIFF */
 	fp = putmail(rp, fdmail, messagefd, timestring, file, dp);
+	if (mmdf_mode)
+	  fputs("\001\001\001\001\n",fp);
+	
 	if (S_ISREG(st.st_mode)) {
-
 #ifdef	USE_NFSMBOX
 	  unlock(file);
 #endif	/* USE_NFSMBOX */
 #ifdef	USE_LOCKF
-	  lseek(fdmail,0,0);
+	  lseek(fdmail,0L,L_SET);
 	  lockf(fdmail, F_ULOCK, 0);/* XX: does this really work?*/
-	  /* Yes it does, but it needs seek to begining of file
-	     first! [mea@utu.fi] */
+	  /* Yes it does, but it needs seek to begining of
+	     the file at first! [mea@utu.fi] */
 #endif	/* USE_LOCKF */
 #ifdef	USE_FLOCK
 	  flock(fdmail, LOCK_UN);
@@ -794,16 +805,23 @@ putmail(rp, fdmail, messagefd, timestring, file, dp)
 	char buf[2];
 	struct stat st;
 	char *fromuser;
+	time_t orig_atime = 0;
+
+	if (keepatime  &&  *(rp->addr->user) != TO_PIPE  &&
+	    stat(file, &st) >= 0)
+	  orig_atime = st.st_atime;
 
 	if ((fp = fdopen(fdmail, "a+")) == NULL) {
 	  notaryreport(NULL,NULL,NULL);
 	  DIAGNOSTIC(rp, EX_TEMPFAIL, "cannot fdopen(%d)", fdmail);
 	  return NULL;
 	}
+
 	/* append From_ line and print out the header */
 	eofindex = ftell(fp);
-	if (*(rp->addr->user) != TO_PIPE && eofindex > 0L &&
-	    fseek(fp, -2L, 2) != -1) {
+	if (!mmdf_mode &&
+	    *(rp->addr->user) != TO_PIPE && eofindex > 0L &&
+	    fseek(fp, -2L, L_XTND) != -1) {
 	  /*
 	   * Determine how many newlines to append to previous text.
 	   *
@@ -815,19 +833,19 @@ putmail(rp, fdmail, messagefd, timestring, file, dp)
 	   * login etc. can distinguish new mail from old.
 	   * The mtime will be set to now by the following write() calls.
 	   */
-	  if (!keepatime || fstat(fileno(fp), &st) < 0)
-	    st.st_atime = 0;
+	  fstat(fileno(fp), &st);
+
 	  len = fread(buf, 1, 2, fp);
-	  if (st.st_atime != 0) {
+	  if (!keepatime) {
 #ifdef	USE_TIMEVAL
 	    struct timeval tv[2];
-	    tv[0].tv_sec = st.st_atime;
+	    tv[0].tv_sec = orig_atime;
 	    tv[1].tv_sec = st.st_mtime;
 	    tv[0].tv_usec = tv[1].tv_usec = 0;
 	    utimes(file, tv);
 #else  /* !USE_TIMEVAL */
 	    struct utimbuf tv;
-	    tv.actime = st.st_atime;
+	    tv.actime  = orig_atime;
 	    tv.modtime = st.st_mtime;
 	    utime(file, &tv);
 #endif /* USE_TIMEVAL */
@@ -835,7 +853,7 @@ putmail(rp, fdmail, messagefd, timestring, file, dp)
 	  if (len == 1 || len == 2) {
 	    --len;
 	    len = (buf[len]!='\n') + (len == 1 ? buf[0]!='\n' : 1);
-	    fseek(fp, 0, 2);	/* to end of file, again */
+	    fseek(fp, 0L, L_XTND);	/* to end of file, again */
 	    if (len > 0 && (fwrite("\n\n", 1, len, fp) != len
 			    || fflush(fp) == EOF)) {
 
@@ -851,20 +869,25 @@ putmail(rp, fdmail, messagefd, timestring, file, dp)
 	      return NULL;
 	    }
 	  } else
-	    fseek(fp, 0L, 2);	/* to end of file, again */
+	    fseek(fp, 0L, L_XTND);	/* to end of file, again */
 	} else if (eofindex > 0L && eofindex < (sizeof "From x\n")) {
 	  /* A mail message *cannot* be this small.  It must be
-	     a corrupted mailbox file.  Ignore them trash bytes. */
-	  fseek(fp, 0L, 0);
+	     a corrupted mailbox file.  Ignore the trash bytes. */
+	  fseek(fp, 0L, L_SET);
 	  eofindex = 0;
 	}
 	if (!(convert_qp && qp_to_8bit(rp)))
 	  convert_qp = 0; /* If malloc failed, clear this too,
 			     if it was not set, it doesn't change.. */
 
+	/* Begin of the file, and MMDF-style ? */
+	if (eofindex == 0 && mmdf_mode == 1 /* Values 2 and 3 exist
+						on PIPEs only.. */   )
+	  fputs("\001\001\001\001\n",fp);
+
 	fromuser = rp->addr->link->user;
 	if (*fromuser == 0) fromuser = "postmaster";
-	if (fprintf(fp, "%s%s %s",
+	if (fprintf(fp, "%s%s  %s",
 		    FROM_,fromuser,timestring) == EOF
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
@@ -877,7 +900,7 @@ putmail(rp, fdmail, messagefd, timestring, file, dp)
 	  fclose(fp);
 	  return NULL;
 	}
-	if (appendlet(rp, fp, file, messagefd) == EOF) {
+	if (appendlet(rp, fp, file, messagefd, is_mime) == EOF) {
 	  /* XX: should I really do this? */
 	  fflush(fp);
 	  ftruncate(fileno(fp), (u_long)eofindex);
@@ -1033,7 +1056,9 @@ program(rp, messagefd, timestring, dp)
 	close(in[1]);
 	errfp = fdopen(in[0], "r");
 	/* write the message */
+	mmdf_mode += 2;
 	fp = putmail(rp, out[1], messagefd, timestring, rp->addr->user+1, dp);
+	mmdf_mode -= 2;
 	if (fp == NULL) {
 	  pid = wait(&status);
 	  close(out[1]);
@@ -1119,6 +1144,23 @@ creatembox(rp, filep, uid, gid, pw)
 	    chown(*filep, *uid, *gid);
 #endif /* USE_FCHOWN */
 	    close(fd);
+	    {
+	      struct stat st;
+#ifdef	USE_TIMEVAL
+	      struct timeval tv[2];
+	      stat(*filep,&st); /* This by all propability will not fail.. */
+	      tv[0].tv_sec = 0;
+	      tv[1].tv_sec = st.st_mtime;
+	      tv[0].tv_usec = tv[1].tv_usec = 0;
+	      utimes(*filep, tv);
+#else
+	      struct utimbuf tv;
+	      stat(*filep,&st); /* This by all propability will not fail.. */
+	      tv.actime  = 0;
+	      tv.modtime = st.st_mtime;
+	      utime(*filep, &tv);
+#endif
+	    }
 	    return 1;
 	  }
 	}
@@ -1167,7 +1209,7 @@ createfile(rp, file, uid)
 	    return fd;
 	  }
 	} else
-		return fd;
+	  return fd;
 
 	/* No system calls in this spot -- must preserve errno */
 	if (TEMPFAIL(saverrno))	/* temporary error? */
@@ -1267,48 +1309,113 @@ setrootuid(rp)
  * appendlet - append letter to file pointed at by fd
  */
 int
-appendlet(rp, fp, file, mfd)
+appendlet(rp, fp, file, mfd, ismime)
 	struct rcpt *rp;
 	FILE *fp;
-	int mfd;
+	int mfd, ismime;
 	char *file;
 {
-	static char buffer[BUFSIZ];
+	static char buffer[BUFSIZ+1];
+	static char *bufend = buffer+BUFSIZ;
 	register int i;
 	register int bufferfull;
+	char iobuf[BUFSIZ];
+	FILE *mfp;
 
-	writebuf((FILE *)NULL, (char *)NULL, 0);	/* initialize state */
+	writebuf((FILE *)NULL, (char *)NULL, 0, 0);	/* initialize state */
 
-	/* can we use cache of message body data */
-	if (readalready != 0) {
-	  if (writebuf(fp, buffer, readalready) != readalready) {
-	    DIAGNOSTIC(rp, EX_IOERR,
-		       "write to \"%s\" failed", file);
-	    return EOF;
+	if (ismime) {
+	  /* can we use cache of message body data */
+	  /* Split it to lines.. */
+	  if (readalready > 0) {
+	    int linelen;
+	    char *s0 = buffer, *s;
+	    while (readalready > 0) {
+	      s = s0;
+	      linelen = 0;
+	      while (*s != 0 && *s != '\n' && readalready > 0) {
+		++s; ++linelen; --readalready;
+	      }
+	      if (*s == '\n' && readalready > 0) {
+		++s; ++linelen; --readalready;
+	      }
+	      if (writebuf(fp, s0, linelen, ismime) != linelen) {
+		DIAGNOSTIC(rp, EX_IOERR,
+			   "write to \"%s\" failed", file);
+		return EOF;
+	      }
+	      s0 = s;
+	    }
+	    rp->status = EX_OK;
+	    return 0;
 	  }
-	  rp->status = EX_OK;
-	  return 0;
-	}
 
-	/* we are assumed to be positioned properly at start of message body */
-	bufferfull = 0;
-	while ((i = read(mfd, buffer, sizeof buffer)) != 0) {
-	  if (i < 0) {
+	  mfp = fdopen(mfd,"r");
+	  setvbuf(mfp, iobuf, _IOFBF, sizeof(iobuf));
+
+#define MFPCLOSE i = dup(mfd); fclose(mfp); dup2(i,mfd); close(i)
+
+	  /* we are assuming to be positioned properly
+	     at the start of the message body */
+	  bufferfull = 0;
+	  readalready = 0;
+	  i = 0;
+	  while ((i = cfgets(buffer, BUFSIZ, mfp)) != EOF) {
+	    /* It MAY be malformed -- if it has a BUFSIZ length
+	       line in it, IT CAN'T BE MIME  :-/		*/
+	    if (i == BUFSIZ && buffer[BUFSIZ-1] != '\n')
+	      ismime = 0;
+	    /* Ok, write the line */
+	    if (writebuf(fp, buffer, i, ismime) != i) {
+	      DIAGNOSTIC(rp, EX_IOERR,
+			 "write to \"%s\" failed", file);
+	      MFPCLOSE;
+	      return EOF;
+	    }
+	  }
+	  if (i == EOF && !feof(mfp)) {
 	    DIAGNOSTIC(rp, EX_IOERR,
 		       "read error from message file", 0);
+	    MFPCLOSE;
 	    return EOF;
 	  }
-	  if (writebuf(fp, buffer, i) != i) {
-	    DIAGNOSTIC(rp, EX_IOERR,
-		       "write to \"%s\" failed", file);
-	    return EOF;
-	  }
-	  readalready = i;
-	  bufferfull++;
-	}
 
-	if (bufferfull > 1)	/* not all in memory, need to reread */
-	  readalready = 0;
+	  MFPCLOSE;
+
+	} else {		/* is NOT MIME message.. */
+
+	  /* can we use cache of message body data */
+	  if (readalready != 0) {
+	    if (writebuf(fp, buffer, readalready, 0) != readalready) {
+	      DIAGNOSTIC(rp, EX_IOERR,
+			 "write to \"%s\" failed", file);
+	      return EOF;
+	    }
+	    rp->status = EX_OK;
+	    return 0;
+	  }
+
+	  /* we are assumed to be positioned properly
+	     at the start of the message body */
+	  bufferfull = 0;
+	  while ((i = read(mfd, buffer, BUFSIZ)) != 0) {
+	    if (i < 0) {
+	      DIAGNOSTIC(rp, EX_IOERR,
+			 "read error from message file", 0);
+	      return EOF;
+	    }
+	    if (writebuf(fp, buffer, i, 0) != i) {
+	      DIAGNOSTIC(rp, EX_IOERR,
+			 "write to \"%s\" failed", file);
+	      return EOF;
+	    }
+	    readalready = i;
+	    bufferfull++;
+	  }
+
+	  if (bufferfull > 1)	/* not all in memory, need to reread */
+	    readalready = 0;
+	}
 	return 0;
 }
 
@@ -1464,11 +1571,11 @@ rbiff(nbp)
 /*
  * Writebuf() is like write(), except any instances of "From " at the
  * beginning of a line cause insertion of '>' at that point.
+ * (Except with MMDF_MODE!)
  */
 
-#if  1 /* Modified code! */
 int
-writebuf(fp, buf, len)
+writebuf(fp, buf, len, ismime)
 	FILE *fp;
 	char *buf;
 	int len;
@@ -1478,55 +1585,109 @@ writebuf(fp, buf, len)
 	int wlen, tlen, i = 0;
 	register char expect;
 	static char save = '\0';
-	static int qp_hex = 0;
-	static int qp_chrs = 0;
 	static char frombuf[8];
 	static char *fromp = NULL;
 
+	static char *buf2 = NULL;
+	static int buf2len = 0;
+
 	if (buf == NULL) {	/* magic initialization */
-	  save = 'F';
-	  qp_hex = 0;
-	  qp_chrs = 0;
+	  save = mmdf_mode ? 0 : 'F';
 	  frombuf[0] = 0;
 	  fromp = frombuf;
 	  return 0;
 	}
+
+#ifdef	USE_ALLOCA
+	buf2 = alloca(len+1);
+#else
+	if (buf2 == NULL) {
+	  buf2 = emalloc(len+1);
+	  buf2len = len;
+	}
+	if (buf2len < len) {
+	  free(buf2);
+	  buf2 = emalloc(len+1);
+	  buf2len = len;
+	}
+#endif
+
+	if (ismime) {
+	  /* ------------------------------------------------------------ */
+	  /* MIME format processing, the input is LINE BY LINE */
+	  if (mime_boundary != NULL && n > mime_boundary_len+2 &&
+	      buf[0] == '-' && buf[1] == '-' &&
+	      strncmp(buf+2,mime_boundary,mime_boundary_len) == 0) {
+	    /* ------------------------------------------------------------ */
+	    /* XX: process MIME boundary! */
+	    i = fwrite(buf, 1, len, fp);
+	    if (i != len) return -1;
+	    return len;
+	  } else if (convert_qp) {
+	    /* ------------------------------------------------------------ */
+	    /* Process Q-P chars of this line */
+	    int qp_hex = 0;
+	    int qp_chrs = 0;
+	    int tlen = 0;
+
+	    for (cp = buf, s = buf2, n = len; n > 0; --n, ++cp) {
+	      register int c = *cp;
+	      /* At first, convert the possible QP character.. */
+	      if (!qp_chrs && c == '=') {
+		qp_chrs = 2;
+		qp_hex  = 0;
+		continue;	/* '=' starts something */
+	      }
+	      if (qp_chrs && c == '\n') {
+		qp_chrs = 0;	/* It was "=[ \t]*\n", which was thrown away.*/
+		continue;
+	      }
+	      if (qp_chrs == 2 && (c == ' ' || c == '\t'))
+		continue; /* It is "=[ \t]*\n", throw it away */
+	      /* Ok, this may be of  =HexHex ? */
+	      if (qp_chrs && ((c >= '0' && c <= '9') ||
+			      (c >= 'A' && c <= 'F') ||
+			      (c >= 'a' && c <= 'f'))) {
+		qp_hex <<= 4;
+		if (c >= '0' && c <= '9') qp_hex += (c - '0');
+		if (c >= 'A' && c <= 'F') qp_hex += (c - 'A' + 10);
+		if (c >= 'a' && c <= 'f') qp_hex += (c - 'a' + 10);
+		--qp_chrs;
+		if (!qp_chrs) c = qp_hex;
+		else continue;	/* Go collect another digit */
+	      } else if (qp_chrs)
+		qp_chrs = 0; /* Failed a HEX check.. */
+	      /* Now we have a converted character at `c', save it! */
+	      *s++ = c;
+	      ++tlen;
+	    }
+	    /* Uhh... Stored the conversion result into buf2[], length  tlen */
+	    if (!mmdf_mode && buf2[0] == 'F' && strncmp(buf2,"From ",5)==0)
+	      putc('>',fp);
+	    i = fwrite(buf2, 1, tlen, fp);
+	    if (i != tlen) return -1;
+	    return tlen;
+	  }
+	  /* ------------------------------------------------------------ */
+	  /* Well, no other processings known.. */
+	  if (!mmdf_mode && buf[0] == 'F' && strncmp(buf,"From ",5)==0)
+	    putc('>',fp);
+	  i = fwrite(buf, 1, len, fp);
+	  if (i != len) return -1;
+	  return len;
+	}
+
+	/* ------------------------------------------------------------ */
+	/* Old-fashioned processing */
+
 	expect = save;
+
 	for (cp = buf, n = len, tlen = 0, s = NULL; n > 0; --n, ++cp) {
 	  register int c = *cp;
 	  ++tlen;
 
-	  /* At first, convert the possible QP character.. */
-	  if (convert_qp) {
-	    if (!qp_chrs && c == '=') {
-	      qp_chrs = 2;
-	      qp_hex  = 0;
-	      continue; /* '=' starts something */
-	    }
-	    if (qp_chrs && c == '\n') {
-	      qp_chrs = 0;  /* It was "=[ \t]*\n", which was thrown away. */
-	      continue;
-	    }
-	    if (qp_chrs == 2 && (c == ' ' || c == '\t'))
-	      continue; /* It is "=[ \t]*\n", throw it away */
-	    /* Ok, this may be of  =HexHex ? */
-	    if (qp_chrs && ((c >= '0' && c <= '9') ||
-			    (c >= 'A' && c <= 'F') ||
-			    (c >= 'a' && c <= 'f'))) {
-	      qp_hex <<= 4;
-	      if (c >= '0' && c <= '9') qp_hex += (c - '0');
-	      if (c >= 'A' && c <= 'F') qp_hex += (c - 'A' + 10);
-	      if (c >= 'a' && c <= 'f') qp_hex += (c - 'a' + 10);
-	      --qp_chrs;
-	      if (!qp_chrs) c = qp_hex;
-	      else continue; /* Go collect another digit */
-	    } else if (qp_chrs)
-	      qp_chrs = 0; /* Failed a HEX check.. */
-	  }
-	  /* Now we have a converted character.. */
-
 	  if (c == '\n') {
-	    expect = 'F';
+	    expect = mmdf_mode ? 0 : 'F';
 	    frombuf[0] = 0;
 	    fromp = frombuf;
 	    if (putc(c,fp) == EOF)
@@ -1569,76 +1730,6 @@ writebuf(fp, buf, len)
 	if (tlen >= 0) return len; /* It worked ok */
 	return -1;	/* errno is correct if tlen == -1 ! */
 }
-#else	/* The original code.. */
-
-int
-writebuf(fp, buf, len)
-	FILE *fp;
-	char *buf;
-	int len;
-{
-	register char *cp, *s;
-	register int n;
-	int wlen, tlen, i = 0;
-	register char expect;
-	static char save = '\0';
-
-	if (buf == NULL) {		/* magic initialization */
-	  save = 'F';
-	  return 0;
-	}
-	expect = save;
-	for (cp = buf, n = len, wlen = tlen = 0, s = NULL; n > 0; --n, ++cp) {
-	  if (*cp == '\n')
-	    expect = 'F';
-	  else if (expect != '\0') {
-	    if (*cp == expect) {
-	      switch (expect) {
-	      case 'F':	expect = 'r'; s = cp; break;
-	      case 'r':	expect = 'o'; break;
-	      case 'o':	expect = 'm'; break;
-	      case 'm':	expect = ' '; break;
-	      case ' ':
-		/* write from start of buffer to s */
-		if (s > buf && tlen >= 0) {
-		  wlen = fwrite(buf,
-				1, s-buf, fp);
-		  if (wlen > 0)
-		    tlen += wlen;
-		  else
-		    tlen = -1;
-		} else if (s == NULL) {
-		  i = sizeof FROM_ -(cp-buf)-2;
-		  fseek(fp, -i, 1);
-		}
-		/* write separator character */
-		if (tlen >= 0) {
-		  save = '>';
-		  fwrite(&save, 1, 1, fp);
-		}
-		expect = '\0';
-		if (s == NULL) {
-		  fwrite(FROM_, 1, i, fp);
-		  s = buf;
-		}
-		/* anticipate future instances */
-		buf = s;
-		break;
-	      }
-	    } else
-	      expect = '\0';
-	  }
-	}
-	save = expect;
-	if (cp > buf && tlen >= 0) {
-	  if ((wlen = fwrite(buf, 1, cp-buf, fp)) > 0)
-	    tlen += wlen;
-	  else
-	    tlen = -1;
-	}
-	return tlen;	/* errno is correct if tlen == -1 ! */
-}
-#endif
 
 
 /* [mea] See if we have a MIME  TEXT/PLAIN message encoded with
@@ -1652,24 +1743,24 @@ qptext_check(rp)
 
 	char **hdrs = *(rp->newmsgheader);
 	char *hdr;
-
 	int cte = 0;
-	int mime = 0;
 
-	if (hdrs == NULL) return 0;
+	is_mime = 0;
 
-	while (hdrs && *hdrs && (!mime || !cte)) {
+	if (hdrs == NULL) return 0; /* Oh ?? */
+
+	while (hdrs && *hdrs && (!is_mime || !cte)) {
 	  hdr = *hdrs;
 	  if (!cte && cistrncmp(hdr,"Content-Transfer-Encoding:",26)==0) {
 	    hdr += 26;
 	    while (*hdr == ' ' || *hdr == '\t') ++hdr;
 	    if (cistrncmp(hdr,"QUOTED-PRINTABLE",16)==0)
 	      cte = 1;
-	  } else if (!mime && cistrncmp(hdr,"MIME-Version:",13)==0)
-	    mime = 1;
+	  } else if (!is_mime && cistrncmp(hdr,"MIME-Version:",13)==0)
+	    is_mime = 1;
 
 	  ++hdrs;
 	}
-	if (!mime) cte = 0;
+	if (!is_mime) cte = 0;
 	return cte;
 }
