@@ -16,6 +16,7 @@
 #include <mail.h>
 #include "malloc.h"
 #include "ta.h"
+#include NDIR_H
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
 #endif	/* USE_UNIONWAIT */
@@ -502,8 +503,9 @@ deliver(mp, startrp, endrp, messagefd, msgoffset, verboselog)
 	  }
 	  if (verboselog)
 	    fprintf(verboselog,"Diagnostic: %s\n",cp);
-	  for (rp = startrp; rp != endrp; rp = rp->next)
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    diagnostic(rp, i, "%s", buf);
+	  }
 	  /* XX: still need to deal with MO_STRIPQUOTES */
 	}
 
