@@ -20,6 +20,7 @@
 #include <sys/file.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include NDIR_H
 
 #include <netdb.h>
 #include <sys/socket.h>
@@ -180,13 +181,11 @@ extern int writebuf __((FILE *fp, char *buf, int len, int downgrade));
 extern int appendlet __((FILE *fp, FILE *verboselog, int mfp, int hsize, int msize, int downgrade));
 extern int smtpopen __((char *host, FILE *fparr[], FILE *verboselog));
 extern int smtpconn __((char *host, FILE *fparr[], FILE *verboselog));
-extern int smtp_ehlo __((FILE *smtpfp[], FILE *verboselog, char *fmt, char *arg));
+extern int smtp_ehlo __((FILE *smtpfp[], FILE *verboselog, char *strbuf));
 extern int ehlo_check __((char *buf));
-/* XX: should use  <stdarg.h> -- on machines where it is available.. */
-/* extern int smtpwrite __((FILE *smtpfp[2], FILE *verboselog, int saverpt, char *fmt, ... )); */
-extern int smtpwrite();
+extern int smtpwrite __((FILE *smtpfp[2], FILE *verboselog, int saverpt, char *linebuf));
 extern int emptyline(), process();
-extern void diagnostics();
+extern void diagnostic();
 extern void notaryreport __((char *rcpt, char *acct, char *status));
 extern void notarystatsave __((char *smtpstatline));
 extern char *notaryacct __((int rc, char *ok_string));
@@ -469,23 +468,31 @@ deliver(smtpfp, verboselog, startrp, endrp, messagefd, msgoffset)
 	int r, nrcpt, hsize, size, tout;
 	int content_kind, flag_downgrade;
 	struct stat stbuf;
-	char MAILFROMstr[80];
+	char SMTPbuf[2000];
 	char *s;
 
 	if (lseek(messagefd, msgoffset, L_SET) < 0L)
 	  warning("Cannot seek to message body! (%m)", (char *)NULL);
-	strcpy(MAILFROMstr,"MAIL From:<%s>");
+	sprintf(SMTPbuf,"MAIL From:<%s>",
+		startrp->addr->link->user);
 	if (ehlo_capabilities & ESMTP_8BITMIME)
-	  strcat(MAILFROMstr, " BODY=8BITMIME");
-	s = MAILFROMstr + strlen(MAILFROMstr);
+	  strcat(SMTPbuf, " BODY=8BITMIME");
+	s = SMTPbuf + strlen(SMTPbuf);
 	/* Size estimate is calculated in the  ctlopen()  by
+
 	   adding msg-body size to the largest known header size,
 	   though excluding possible header and body rewrites.. */
-	if (ehlo_capabilities & ESMTP_SIZEOPT)
+	if (ehlo_capabilities & ESMTP_SIZEOPT) {
 	  sprintf(s," SIZE=%d",startrp->desc->msgsizeestimate);
+	  s += strlen(s);
+	}
+	if ((ehlo_capabilities & ESMTP_DSN) &&
+	    rp->desc->envid != NULL) {
+	  sprintf(s," ENVID=%s",rp->desc->envid);
+	  s += strlen(s);
+	}
 
-	r = smtpwrite(smtpfp, verboselog, 1, MAILFROMstr,
-		      startrp->addr->link->user);
+	r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	if (r != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),NULL);
@@ -496,7 +503,31 @@ deliver(smtpfp, verboselog, startrp, endrp, messagefd, msgoffset)
 	}
 	nrcpt = 0;
 	for (rp = startrp; rp != endrp; rp = rp->next) {
-	  r = smtpwrite(smtpfp, verboselog, 1, "RCPT To:<%s>", rp->addr->user);
+	  sprintf(SMTPbuf, "RCPT To:<%s>", rp->addr->user);
+	  s = SMTPbuf + strlen(SMTPbuf);
+
+	  if (ehlo_capabilities & ESMTP_DSN) {
+	    if (rp->dsnflags & _DSN_RET_NO)
+	      strcat(s, " RET=NO");
+	    else if (rp->dsnflags & _DSN_RET_YES)
+	      strcat(s, " RET=YES");
+
+	    if (rp->dsnflags & _DSN_NOTIFY_SUCCESS)
+	      strcat(s, " NOTIFY=SUCCESS");
+	    else if (rp->dsnflags & _DSN_NOTIFY_FAILURE)
+	      strcat(s, " NOTIFY=FAILURE");
+	    else if (rp->dsnflags & _DSN_NOTIFY_ALWAYS)
+	      strcat(s, " NOTIFY=ALWAYS");
+	    else if (rp->dsnflags & _DSN_NOTIFY_NEVER)
+	      strcat(s, " NOTIFY=NEVER");
+
+	    s += strlen(s);
+	    if (rp->orcpt != NULL) {
+	      sprintf(s, " ORCPT=%s", rp->orcpt);
+	    }
+	  }
+	  
+	  r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	  if (r != EX_OK) {
 	    notaryreport(rp->addr->user, notaryacct(r,"*NOTOK"),NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
@@ -613,7 +644,7 @@ deliver(smtpfp, verboselog, startrp, endrp, messagefd, msgoffset)
 	for (rp = startrp; rp != endrp; rp = rp->next)
 	  if (rp->status == EX_OK) {
 	    notaryreport(rp->addr->user,
-			 notaryacct(r,"relayed"), /* XX: NOTARY DRPT ?? */
+			 notaryacct(r,"relayed"), /* XXX: NOTARY DRPT ?? */
 			 NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
 	  }
@@ -875,6 +906,7 @@ smtpopen(host, fparr, verboselog)
 	FILE *fparr[], *verboselog;
 {
 	int i;
+	char SMTPbuf[1000];
 
 	if (debug) {
 	  fprintf(logfp, "%d#\tsmtpopen: connecting to %s\n", pid, host);
@@ -888,7 +920,8 @@ smtpopen(host, fparr, verboselog)
 	  if (esmtp_on_banner) {
 	    /* Either it is not tested, or it is explicitely
 	       desired to be tested, and was found! */
-	    i = smtp_ehlo(fparr, verboselog, "EHLO %s", myhostname);
+	    sprintf(SMTPbuf, "EHLO %s", myhostname);
+	    i = smtp_ehlo(fparr, verboselog, SMTPbuf);
 	    if (i == EX_TEMPFAIL) {
 	      /* There are systems, which hang up on us
 		 when we present "EHLO".. Do normal HELO.. */
@@ -902,8 +935,10 @@ smtpopen(host, fparr, verboselog)
 	      fprintf(verboselog,
 		      "  EHLO responce flags = 0x%02x\n",
 		      ehlo_capabilities);
-	  } else
-	    i = smtpwrite(fparr, verboselog, 1, "HELO %s", myhostname);
+	  } else {
+	    sprintf(SMTPbuf, "HELO %s", myhostname);
+	    i = smtpwrite(fparr, verboselog, 1, SMTPbuf);
+	  }
 	} while (i == EX_TEMPFAIL && firstmx <= mxcount);
 	if (debug) {
 	  fprintf(logfp, "%d#\tsmtpopen: status = %d\n", pid, i);
@@ -1323,34 +1358,17 @@ int dflag = 0;
 int within_ehlo = 0;
 
 /* VARARGS */
-/*int
-smtpwrite(smtpfp, verboselog, saverpt, fmt, arg)
+int
+smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	FILE *smtpfp[2], *verboselog;
 	int saverpt;
-	char *fmt, *arg;		*/
-int
-smtpwrite(va_alist)
-	va_dcl
+	char *strbuf;
 {
-	va_list ap;
-	FILE **smtpfp;
-	int saverpt;
-	FILE *verboselog;
-	char *fmt;
-	int argc = 0;
-	unsigned long argv[20];
 	register char *s, *cp;
 	int i, response, r, infd, outfd;
-	char ch, buf[8096];	/* XX: growl */
 	char *se;
-
-	&smtpfp; /* Just make sure it stays in memory .. */
-
-	va_start(ap);
-	smtpfp     = va_arg(ap, FILE **);
-	verboselog = va_arg(ap, FILE *);
-	saverpt    = va_arg(ap, int);
-	fmt	   = va_arg(ap, char *);
+	char buf[8192]; /* XX: Eh... Static buffer.. */
+	char ch;
 
 #ifdef	TIMEOUT
 	alarm(timeout);	/* This much total to write and get answer */
@@ -1362,42 +1380,16 @@ smtpwrite(va_alist)
 	infd = fileno(smtpfp[0]);
 	outfd = fileno(smtpfp[1]);
 
-	if (fmt == NULL)
-	  va_end(ap);
-	else {
-	  /* we use raw I/O so we have control over what happens on
-	     interrupts, etc. -- I don't like setjmp/longjmp */
-	  for (s = fmt, cp = buf; *s != '\0'; ++s) {
-	    if (*s == '%') {
-	      switch (*++s) {
-	      case 's':
-		argv[argc] = (unsigned long)va_arg(ap, char*);
-		strcpy(cp, (char*)argv[argc]);
-		++argc;
-		cp += strlen(cp);
-		break;
-	      case 'd':
-		argv[argc] = (unsigned long)va_arg(ap, int);
-		sprintf(cp, "%d", (int)argv[argc]);
-		++argc;
-		cp += strlen(cp);
-		break;
-	      default:
-		*cp++ = '%';
-	      case '%':
-		*cp++ = *s;
-		break;
-	      }
-	    } else 
-	      *cp++ = *s;
-	  }
-	  *cp++ = '\r';
-	  *cp++ = '\n';
-	  va_end(ap);
-	  if (verboselog)
-	    fwrite(buf, cp - buf, 1, verboselog);
+	if (strbuf != NULL) {
+	  int len = strlen(strbuf) + 2;
 
-	  if ((r = write(outfd, buf, cp - buf)) == -1) {
+	  memcpy(buf,strbuf,len-2);
+	  memcpy(buf+len-2,"\r\n",2);
+
+	  if (verboselog)
+	    fwrite(buf, len, 1, verboselog);
+
+	  if ((r = write(outfd, buf, len)) == -1) {
 #ifdef	TIMEOUT
 	    if (gotalarm) {
 	      strcpy(remotemsg, "Timeout on cmd write");
@@ -1412,8 +1404,8 @@ smtpwrite(va_alist)
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    return EX_TEMPFAIL;
-	  } else if (r != cp - buf) {
-	    sprintf(remotemsg, "500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, cp - buf);
+	  } else if (r != len) {
+	    sprintf(remotemsg, "500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
 	    notaryreport(NULL,NULL,remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
@@ -1421,24 +1413,18 @@ smtpwrite(va_alist)
 	  }
 	  if (logfp != NULL) {
 	    if (dflag) abort();
-	    fprintf(logfp, "%dw\t", pid);
-	    fprintf(logfp, fmt,
-		    argv[0],argv[1],argv[2],
-		    argv[3],argv[4],argv[5]);
-	    fprintf(logfp, "\n");
+	    fprintf(logfp, "%dw\t%s\n", pid, strbuf);
 	    fflush(logfp);
 	    dflag = 1;
 	  }
 	}
 	i = 2;	/* state variable, beginning of new line */
 
-	if (fmt) {
-	  if (strncmp(fmt,"RSET",4) != 0) /* If RSET, append to previous! */
+	if (strbuf) {
+	  if (strncmp(strbuf,"RSET",4) != 0) /* If RSET, append to previous! */
 	    *remotemsg = 0;
 	  rmsgappend("\r<<- ");
-	  rmsgappend(fmt,
-		     argv[0],argv[1],argv[2],
-		     argv[3],argv[4],argv[5]);
+	  rmsgappend("%s", strbuf);
 	} else {
 	  strcpy(remotemsg,"\r<<- (null)");
 	}
@@ -1447,10 +1433,8 @@ smtpwrite(va_alist)
 	  fprintf(logfp, "%d#\tAttempting to read reply\n",pid);
 	  fflush(logfp);
 	}
-	if (statusreport && fmt != NULL) {
-	  report(fmt,
-		 argv[0],argv[1],argv[2],
-		 argv[3],argv[4],argv[5]);
+	if (statusreport && strbuf != NULL) {
+	  report("%s", strbuf);
 	}
 	cp = buf;
 	do {
@@ -1467,7 +1451,7 @@ smtpwrite(va_alist)
 		*s = '\0';
 		if (within_ehlo)
 		  ehlo_check(&buf[4]);
-		if (!fmt && !esmtp_on_banner)
+		if (!strbuf && !esmtp_on_banner)
 		  esmtp_banner_check(&buf[4]);
 		if (logfp != NULL) {
 		  if (debug)
@@ -1519,14 +1503,14 @@ smtpwrite(va_alist)
 #ifdef	TIMEOUT
 	    if (gotalarm) {
 	      sprintf(remotemsg, "466 (Timeout on %sSMTP responce read, Cmd: %s)",
-		      fmt == NULL ? "initial ":"", fmt);
+		      strbuf == NULL ? "initial ":"", strbuf);
 	      notaryreport(NULL,NULL,remotemsg);
 	    } else
 #endif /* TIMEOUT */
 	      {
 		se = strerror(errno);
 		sprintf(remotemsg, "500 (Error on %sSMTP responce read: %s, Cmd: %s)",
-			fmt == NULL ? "initial ":"", se, fmt);
+			strbuf == NULL ? "initial ":"", se, strbuf);
 		notaryreport(NULL,NULL,remotemsg);
 	      }
 	    dflag = 0;
@@ -1535,7 +1519,7 @@ smtpwrite(va_alist)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    return EX_TEMPFAIL;
 	  } else {
-	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",fmt);
+	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",strbuf);
 	    notaryreport(NULL,NULL,remotemsg);
 	    dflag = 0;
 	    if (verboselog)
@@ -1584,7 +1568,7 @@ smtpwrite(va_alist)
 
 	if (within_ehlo)
 	  ehlo_check(&buf[4]);
-	if (!fmt && !esmtp_on_banner)
+	if (!strbuf && !esmtp_on_banner)
 	  esmtp_banner_check(&buf[4]);
 
 	rmsgappend("\r->> %s",buf);
@@ -1641,14 +1625,14 @@ smtpwrite(va_alist)
 
 
 int
-smtp_ehlo(smtpfp, verboselog, fmt, arg)
+smtp_ehlo(smtpfp, verboselog, strbuf)
 	FILE *smtpfp[2], *verboselog;
-	char *fmt, *arg;
+	char *strbuf;
 {
 	int rc;
 	within_ehlo = 1;
 	ehlo_capabilities = 0;
-	rc = smtpwrite(smtpfp, verboselog, 1, fmt, arg);
+	rc = smtpwrite(smtpfp, verboselog, 1, strbuf);
 	within_ehlo = 0;
 	return rc;
 }
